@@ -20,9 +20,6 @@ limitations under the License.
 
 # flake8: noqa: F401
 
-from .exceptions import (
-    InterfaceNotFound, LabNotFound, LinkNotFound,
-    NodeNotFound
-)
+from .exceptions import InterfaceNotFound, LabNotFound, LinkNotFound, NodeNotFound
 from .virl2_client import ClientLibrary, InitializationError
 from .models.authentication import Context
