@@ -77,8 +77,8 @@ class ClPyats:
 
         testbed_yaml = self._lab.get_pyats_testbed()
         data = loader.load(io.StringIO(testbed_yaml))
-        data.devices.terminal_server.connections.cli.username = username
-        data.devices.terminal_server.connections.cli.password = password
+        data.devices.terminal_server.credentials.default.username = username
+        data.devices.terminal_server.credentials.default.password = password
         self._testbed = data
 
     def run_command(self, node_label, command):
